#!/usr/bin/env ruby
# = RubyLogo
# === A Logo interpreter written in Ruby
# Christian Kandler
# mailto:ckand002@student.informatik.fh-wiesbaden.de
#

# LogoTurtle provides turtle graphics objects which manage the status of a
# drawing turtle and methods to control it. LogoTurtle objects do not actively
# communicate with other components but store their status information until it
# is requested. All messages sent to the LogoTurtle and all actions invoked
# through its methods are stored in a list. All actually drawn lines are also
# stored seperately.
#
# Anything is allowed as 'color' attribute. The LogoTurtle can not determine if
# a string is a valid color description or not. It is up to the user interface
# to evaluate (or ignore) the color attribute according to its capabilities.
# The color attributes can be set directly. This should only be used by the
# user interface to correct an invalid entry with a valid default value. Normal
# color changes should be initiated through the LogoTurtle#setColor and
# LogoTurtle#setBackground methods to include the change in the message log.
#
class LogoTurtle

  # Container for the attributes which define a line drawn by the LogoTurtle.
  #
  Line = Struct.new("Line", :startx, :starty, :endx, :endy, :color)

  # Current color of the drawing pen [String]
  attr_accessor :color
  # Current background color of the drawing area [String]
  attr_accessor :background
  # Current position on the horizontal plane, relative to starting position [Float]
  attr_reader :posx
  # Current position on the vertical plane, relative to starting position [Float]
  attr_reader :posy
  # Current heading in degrees, relative to the starting heading [Float]
  attr_reader :direction
  # Pen connected to the drawing area [Boolean]
  attr_reader :pen
  # Turtle visible on drawing area [Boolean]
  attr_reader :visible
  # List of lines drawn by the turtle [Array]
  attr_reader :lines
  # List of status messages recorded by the turtle [Array]
  attr_reader :messages
  # Converts degrees to radians
  DEG = Math::PI / 180

  # Return an initialized LogoTurtle instance with specified starting attributes.
  #
  # <b>Parameters:</b>
  #
  # <i>posx:</i> The turtle's position on the horizontal plane [Float]
  #
  # <i>posy:</i> The turtle's position on the vertical plane [Float]
  #
  # <i>direction:</i> The turtle's heading in degrees [Float]
  #
  # <i>color:</i> The color of the drawing pen [String]
  #
  # <i>background:</i> Background color of the drawing area [String]
  #
  # <b>Returns:</b>
  #
  # The initialized LogoTurtle [LogoTurtle]
  #
  def initialize(posx=0.0, posy=0.0, direction=0.0, color="black", background="white")
    @posx, @posy, @direction = posx.to_f, posy.to_f, direction.to_f
    @color, @background, @pen, @visible = color.to_s, background.to_s, true, true
    @lines, @messages = [], []
    @homeposx, @homeposy, @homedir, @homecol = @posx, @posy, @direction, @color
  end

  # Move the turtle for the given distance in the direction it's currently heading.
  #
  # <b>Parameters:</b>
  #
  # <i>distance:</i> Distance to be travelled by the turtle [Float]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def forward(distance)
    newx = @posx + distance.to_f * Math.sin(@direction * DEG)
    newy = @posy + distance.to_f * Math.cos(@direction * DEG)
    @lines << Line.new(@posx, @posy, newx, newy, @color) if @pen
    @posx, @posy = newx, newy
    log "MOVE FORWARD #{distance}"
  end

  # Move the turtle for the given distance in the opposite direction than the
  # one it's currently heading.
  #
  # <b>Parameters:</b>
  #
  # <i>distance:</i> Distance to be travelled by the turtle [Float]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def backward(distance)
    forward(-distance.to_f)
  end

  # Turn the turtle for the given amount of degrees to the left.
  #
  # <b>Parameters:</b>
  #
  # <i>degrees:</i> Amount of degrees by which the turtle will turn [Float]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def left(degrees)
    @direction = normalize(@direction - degrees.to_f)
    log "TURN LEFT #{degrees % 360}"
  end

  # Turn the turtle for the given amount of degrees to the right.
  #
  # <b>Parameters:</b>
  #
  # <i>degrees:</i> Amount of degrees by which the turtle will turn [Float]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def right(degrees)
    @direction = normalize(@direction + degrees.to_f)
    log "TURN RIGHT #{degrees % 360}"
  end

  # Raise the pen from the drawing area. Moving the turtle will not draw lines.
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def penup
    @pen = false
    log "STATUS: PEN UP"
  end

  # Lower the pen onto the drawing area. Moving the turtle will draw lines.
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def pendown
    @pen = true
    log "STATUS: PEN DOWN"
  end

  # Make the turtle invisible. It can still perform all its actions and draw lines.
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def hide
    @visible = false
    log "STATUS: HIDE TURTLE"
  end

  # Make the turtle visible.
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def show
    @visible = true
    log "STATUS: SHOW TURTLE"
  end

  # Assign a new drawing color to the turtle. Validity of the given color can
  # not be confirmed by the turtle, it depends on the capabilities of the
  # interface which implements the turtle.
  #
  # <b>Parameters:</b>
  #
  # <i>color:</i> The new drawing color [String]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def setColor(color)
    @color = color
    log "STATUS: COLOR #{color}"
  end

  # Assign a new background color to the turtle. Validity of the given color can
  # not be confirmed by the turtle, it depends on the capabilities of the
  # interface which implements the turtle.
  #
  # <b>Parameters:</b>
  #
  # <i>color:</i> The new background color [String]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def setBackground(color)
    @background = color
    log "STATUS: BACKGROUND #{color}"
  end

  # Move the turtle directly from its current position to the given coordinates.
  # This will draw a line (unless the pen is up) but not change the turtle's
  # heading.
  #
  # <b>Parameters:</b>
  #
  # <i>x:</i> The new position on the horizontal plane [Float]
  #
  # <i>y:</i> The new position on the vertical plane [Float]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def move(x, y)
    newx = @homeposx + x.to_f
    newy = @homeposy + y.to_f
    @lines << Line.new(@posx, @posy, newx, newy, @color) if @pen
    @posx, @posy = newx, newy
    log "MOVE TO POSITION [#{x}, #{y}]"
  end

  # Move the turtle directly from its current position to it's home position.
  # (the position the turtle had during initialization). This will draw a line
  # (unless the pen is up) but not change the turtle's heading.
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def home
    move(0, 0)
  end

  # Turn the turtle into the given absolute direction. 0 (=360) represents
  # the direction during initialization. Positive numbers represent clockwise
  # rotation.
  #
  # <b>Parameters:</b>
  #
  # <i>direction:</i> Direction to which the turtle will turn [Float]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def turn(direction)
    @direction = normalize(@homedir + direction.to_f)
    log "TURN TO #{@direction}"
  end

  # Put the turtle back to it's starting state and delete all past actions
  # from the log.
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def reset
    @lines.clear
    @messages.clear
    @posx, @posy, @direction = @homeposx, @homeposy, @homedir
    @color, @pen, @visible = @homecol, true, true
    log "STATUS: RESET TURTLE"
  end

  # Write the given message into the turtle's log. This method is used for status
  # messages of movement methods, syntax errors from a parser and other outside
  # information the turtle should, tell to an object requesting it's actions.
  #
  # <b>Parameters:</b>
  #
  # <i>message:</i> Message to be appended to the log [String]
  #
  # <b>Returns:</b>
  #
  # The latest entry in the turtle's message log [String]
  #
  def log(message)
    @messages << message.to_s
    message.to_s
  end

  # Delete all messages from the log but keep the current state of the turtle
  # and all drawn lines
  #
  # <b>Returns:</b>
  #
  # The empty log [Array]
  #
  def clearMessages
    @messages.clear
  end

  # calculates a direction between 0 and 360 degrees which is equivalent to the
  # given direction.
  #
  # <b>Parameters:</b>
  #
  # <i>direction:</i> Direction to be normalized [Float]
  #
  # <b>Returns:</b>
  #
  # An equivalent direction between 0 and 360 [Float]
  #
  def normalize(direction)
    while direction.to_f < 0 do direction += 360 end
    direction % 360
  end

  # call <i>block</i> for each line to be drawn, passing the Line object as a
  # parameter
  #
  # <b>Returns:</b>
  #
  # A Container with the attributes of the line [Line]
  #
  def getLines
    lines.each { |line| yield line }
  end

  # returns an array representation of the LogoTurtle instance
  #
  # <b>Returns:</b>
  #
  # An array containing all attributes of the LogoTurtle's current state [Array]
  #
  def to_a
    return [@posx, @posy, @direction, @color, @background, @pen, @visible,
      @lines, @messages]
  end

  # returns a string representation of the LogoTurtle
  #
  # <b>Returns:</b>
  #
  # A semicolon seperated string containing all attributes of the LogoTurtle's
  # current state [String]
  #
  def to_s
    return [@posx, @posy, @direction, @color, @background, @pen, @visible,
      @lines.join(", "), @messages.join(", ")].join(";")
  end

  alias fd forward
  alias bk backward
  alias lt left
  alias rt right
  alias pu penup
  alias pd pendown
end



# Ignore tests unless file is run directly
if $0 == __FILE__
  require 'test/unit'
  class TestLogoTurtle < Test::Unit::TestCase

    # Constant delta for approximative testing
    DELTA = 0.000000000001

    # Test initialization of LogoTurtle instances
    #
    def testinit
      a = LogoTurtle.new()
      assert_equal(0, a.posx)
      assert_equal(0, a.posy)
      assert_equal(0, a.direction)
      assert_equal("black", a.color)
      assert_equal("white", a.background)
      b = LogoTurtle.new(-2.5, -0.5, -45, "anycolor", "anyothercolor")
      assert_equal(-2.5, b.posx)
      assert_equal(-0.5, b.posy)
      assert_equal(-45, b.direction)
      assert_equal("anycolor", b.color)
      assert_equal("anyothercolor", b.background)
      c = LogoTurtle.new("a", "b", "c", 1, [2, 3])
      assert_equal(0.0, c.posx)
      assert_equal(0.0, c.posy)
      assert_equal(0.0, c.direction)
      assert_equal("1", c.color)
      assert_equal("23", c.background)
    end

    # Test movement methods for LogoTurtle instances
    #
    def testmovement
      a = LogoTurtle.new(-100.0, -100.0, 45.0)
      a.right(495)
      assert_in_delta(180.0, a.direction, DELTA)
      a.left(-90)
      assert_in_delta(270.0, a.direction, DELTA)
      a.forward(100)
      assert_in_delta(-200.0, a.posx, DELTA)
      assert_in_delta(-100.0, a.posy, DELTA)
      a.left(90+360+360)
      assert_in_delta(180.0, a.direction, DELTA)
      a.backward(100)
      assert_in_delta(-200.0, a.posx, DELTA)
      assert_in_delta(0, a.posy, DELTA)
      a.move(-12.34, 56.78)
      assert_in_delta(-112.34, a.posx, DELTA)
      assert_in_delta(-43.22, a.posy, DELTA)
      a.turn(45)
      assert_in_delta(90.0, a.direction, DELTA)
      a.home
      assert_in_delta(-100.0, a.posx, DELTA)
      assert_in_delta(-100.0, a.posy, DELTA)
      b = LogoTurtle.new
      b.right("asdf")
      b.forward("asdf")
      b.left("asdf")
      b.backward("asdf")
      assert_in_delta(0.0, b.posx, DELTA)
      assert_in_delta(0.0, b.posy, DELTA)
      assert_in_delta(0.0, b.direction, DELTA)
    end

    # Test state changing methods for LogoTurtle instances
    #
    def teststate
      a = LogoTurtle.new
      assert(a.pen)
      assert(a.visible)
      assert_equal([], a.lines)
      assert_equal([], a.messages)
      a.penup
      assert(!a.pen)
      a.forward(100)
      assert_equal(0, a.lines.length)
      a.pendown
      assert(a.pen)
      a.forward(100)
      assert_equal(1, a.lines.length)
      a.hide
      assert(!a.visible)
      a.show
      assert(a.visible)
      assert_equal(6, a.messages.length)
      a.clearMessages
      assert_equal(0, a.messages.length)
      assert_equal(1, a.lines.length)
      a.setColor("invalidcolor")
      assert_equal("invalidcolor", a.color)
      a.color = "red" # simulate override from user interface
      assert_equal("red", a.color)
      a.setBackground("invalidbackground")
      assert_equal("invalidbackground", a.background)
      a.background = "grey" # simulate override from user interface
      assert_equal("grey", a.background)
      assert_equal(2, a.messages.length)
      assert_equal(1, a.lines.length)
      a.log("test abc test")
      a.log(12345)
      assert_equal(4, a.messages.length)
      a.reset
      assert_equal("black", a.color)
      assert_equal(1, a.messages.length)
      assert_equal(0, a.lines.length)
    end

    # Test methods to retrieve lines and to convert instances
    #
    def testget
      a = LogoTurtle.new
      a.forward(200); a.right(90); a.forward(200)
      loopcount = 0
      a.getLines { |line| assert_equal("black", line.color); loopcount += 1 }
      assert_equal(2, loopcount)
      assert_equal([0.0, 0.0, 0.0, "black", "white", true, true, [], []],
        LogoTurtle.new.to_a)
      assert_equal("0.0;0.0;0.0;black;white;true;true;;", LogoTurtle.new.to_s)
    end

    # Test private methods used by LogoTurtle instances
    #
    def testprivate
      assert_in_delta(0, LogoTurtle.new().normalize(0), DELTA)
      assert_in_delta(0, LogoTurtle.new().normalize(360), DELTA)
      assert_in_delta(45, LogoTurtle.new().normalize(405), DELTA)
      assert_in_delta(350, LogoTurtle.new().normalize(-10), DELTA)
      assert_in_delta(220, LogoTurtle.new().normalize(-500), DELTA)
      assert_in_delta(90, LogoTurtle.new().normalize(36000000000090.0), DELTA)
      assert_in_delta(0, LogoTurtle.new().normalize("asdf"), DELTA)
    end
  end
end


