#!/usr/bin/env ruby
# = RubyLogo
# === A Logo interpreter written in Ruby
# Christian Kandler
# mailto:ckand002@student.informatik.fh-wiesbaden.de
#

begin
  require 'LogoTurtle'
  require 'LogoParser'
  require 'LogoGUIInfoBox'
  require 'fox14'
rescue LoadError => error
  puts "#{error}\nOne of the required files is missing. Make sure that the \
classes \'LogoTurtle\', \'LogoParser\' and \'LogoGUIInfoBox\' are available"
  exit 1
end
include Fox

# The default RubyLogo application graphical user interface. It provides the
# following interface components:
# - a drawing area for a Turtle object
# - a text window in which logo programs can be edited
# - an input field to execute drawing commands immediately when they are entered
# - a terminal window which displays errors encountered by the code parser
# - a button bar which offers options to load, save and execute Logo programs
# - a status line providing information about the current state of the drawing
#   turtle
#
# The interface initializes a Turtle and an associated Parser and manages the
# communication between its input/output components and these modules.
#
class LogoGUI < FXMainWindow
  # Converts degrees to radians
  DEG = Math::PI / 180

  # Initialize a new RubyLogo application window and its components.
  #
  def initialize(app)
    super(app, "RubyLogo", geticon(app, "icon"), nil, DECOR_ALL, 50, 50, 1000, 650)
    FXToolTip.new(app)

    # Menu bar and buttons at the top of the main window
    menubar = FXMenuBar.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,
      0,0,0,0,0,0,0,0, 20)

    pushNew = FXButton.new(menubar, "&New\t\tCreate new logo script",
      geticon(app, "new"), nil, 0, FRAME_RAISED|BUTTON_TOOLBAR|ICON_BEFORE_TEXT)
    pushNew.connect(SEL_COMMAND, method(:btnNew))
    pushNew.tipText = "New"
    pushNew.backColor = FXRGB(205,205,205)

    pushLoad = FXButton.new(menubar, "&Load\t\tLoad logo script",
      geticon(app, "load"), nil, 0, FRAME_RAISED|BUTTON_TOOLBAR|ICON_BEFORE_TEXT)
    pushLoad.connect(SEL_COMMAND, method(:btnLoad))
    pushLoad.tipText = "Load Script"
    pushLoad.backColor = FXRGB(205,205,205)

    pushSave = FXButton.new(menubar, "&Save\t\tSave logo script",
      geticon(app, "save"), nil, 0, FRAME_RAISED|BUTTON_TOOLBAR|ICON_BEFORE_TEXT)
    pushSave.connect(SEL_COMMAND, method(:btnSave))
    pushSave.tipText = "Save Script"
    pushSave.backColor = FXRGB(205,205,205)

    pushRun = FXButton.new(menubar, "&Run\t\tRun logo script",
      geticon(app, "run"), nil, 0, FRAME_RAISED|BUTTON_TOOLBAR|ICON_BEFORE_TEXT)
    pushRun.connect(SEL_COMMAND, method(:btnRun))
    pushRun.tipText = "Run"
    pushRun.backColor = FXRGB(205,205,205)

    pushExit = FXButton.new(menubar, "&Exit\t\tExit program", geticon(app, "exit"),
      nil, 0, FRAME_RAISED|BUTTON_TOOLBAR|LAYOUT_RIGHT|ICON_BEFORE_TEXT)
    pushExit.connect(SEL_COMMAND) {exit}
    pushExit.tipText = "Exit"
    pushExit.backColor = FXRGB(205,205,205)

    pushHelp = FXButton.new(menubar, "&Help\t\tShow help page", geticon(app, "help"),
      nil, 0, FRAME_RAISED|BUTTON_TOOLBAR|LAYOUT_RIGHT|ICON_BEFORE_TEXT)
    pushHelp.connect(SEL_COMMAND, method(:btnHelp))
    pushHelp.tipText = "Help"
    pushHelp.backColor = FXRGB(205,205,205)

    # Status bar at the bottom of the main window
    @statusbar = FXStatusBar.new(self, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X)
    @statusbar.statusLine.normalText = "RubyLogo"
    @turtleinfo = FXLabel.new(@statusbar, "", nil,
      LAYOUT_RIGHT|LAYOUT_FILL_Y|FRAME_SUNKEN, 0, 0)

    # Layout composites for the main area of the user interface
    splith = FXSplitter.new(self,
      LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_HORIZONTAL|SPLITTER_TRACKING, 0, 0)
    splitv = FXSplitter.new(splith,
      LAYOUT_FILL_X|SPLITTER_VERTICAL|SPLITTER_TRACKING|SPLITTER_REVERSED, 0, 0, 400)
    editbox = FXPacker.new(splitv, LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0)

    # Input line which sends entered Logo commands immediately to the parser
    @cmdline = FXTextField.new(editbox, 0, nil, 0,
      LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|FRAME_GROOVE|TEXTFIELD_ENTER_ONLY)
    @cmdline.backColor = FXRGB(255, 255, 255)
    @cmdline.textColor = FXRGB(0, 0, 0)
    @cmdline.connect(SEL_COMMAND, method(:lineEval))

    # Editor window for Logo programs
    @editor = FXText.new(editbox, nil, 0,
      LAYOUT_FILL_X|LAYOUT_FILL_Y|VSCROLLER_ALWAYS)
    @editor.backColor = FXRGB(255, 255, 255)
    @editor.textColor = FXRGB(0, 0, 0)
    @editor.tabColumns = 2

    # Read-only terminal area for status and error messages
    @terminal = FXText.new(splitv, nil, 0,
      LAYOUT_FILL_X|VSCROLLER_ALWAYS|TEXT_READONLY, 0, 0, 400, 100)
    @terminal.backColor = FXRGB(50, 50, 50)
    @terminal.textColor = FXRGB(0, 255, 0)
    @terminal.tabColumns = 2

    # Drawing area for the turtle
    @scrollwin = FXScrollWindow.new(splith, LAYOUT_FILL_X|LAYOUT_FILL_Y)
    scrollfix = FXPacker.new(@scrollwin, LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    # scrollwin has wrong range associated to its scrollbars
    # manually update to known real values
    @scrollwin.verticalScrollBar.range = 5000
    @scrollwin.horizontalScrollBar.range = 5000

    replaceTurtle # create new turtle for the drawing area
    @turtlearea = FXCanvas.new(scrollfix, nil, 0,
      LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 5000, 5000)
    @turtlearea.connect(SEL_PAINT) { |sender, sel, event|
      FXDCWindow.new(@turtlearea, event) { |terrarium|
        # drawing area background
        terrarium.foreground = @tortoise.background
        terrarium.fillRectangle(event.rect.x, event.rect.y, event.rect.w, event.rect.h)
        # lines drawn by the turtle
        @tortoise.getLines { |line|
          terrarium.foreground = line.color # defaults to 'black' if not a valid color
          begin
            # move coordinates to adapt to drawing area
            terrarium.drawLine(line.startx+2500, 2500-line.starty,
              line.endx+2500, 2500-line.endy)
          rescue RangeError
            @tortoise.log "ERROR: Drawing coordinates out of range"
            writeLog @tortoise.messages
          end
        }
        drawTurtle(terrarium)         # draw turtle at its current position
      }
    }
    refreshTurtle                     # update status for turtle
  end

  # Action when the button 'New' is pushed.
  # Reset the program to its startup state.
  # Replace Turtle and Parser with new instances.
  #
  def btnNew(sender, sel, event)
    @editor.setText("")
    replaceTurtle
    refreshTurtle
  end

  # Action when the button 'Load' is pushed.
  # Open a file browser and load the selected file into the editor area.
  #
  def btnLoad(sender, sel, event)
    openDialog = FXFileDialog.new(self, "Load Script")
    openDialog.patternList = ["Logo (*.logo)", "All Files (*)"]
    if openDialog.execute(PLACEMENT_SCREEN) != 0
      btnNew(sender, sel, event) # clear all before loading new file
      begin
        File.open(openDialog.filename) { |file| @editor.setText(file.read) }
      rescue StandardError => error
        FXMessageBox.error(self, MBOX_OK, "Error", "#{error}")
      end
    end
    return 1
  end

  # Action when the button 'Save' is pushed.
  # Open a file browser and save the contents of the editor area to the entered file.
  #
  def btnSave(sender, sel, event)
    saveDialog = FXFileDialog.new(self, "Save Script")
    saveDialog.patternList = ["Logo (*.logo)", "All Files (*)"]
    if saveDialog.execute(PLACEMENT_SCREEN) != 0
      if File.exists? saveDialog.filename
        if MBOX_CLICKED_NO == FXMessageBox.warning(self, MBOX_YES_NO,
          "Overwrite File?", "File #{saveDialog.filename} already exists. Overwrite?")
          return 1
        end
      end
      begin
        File.open(saveDialog.filename, "w") { |file| file.write(@editor.text) }
      rescue Errno::ENOSPC => error
        File.delete(saveDialog.filename) if File.exist?(saveDialog.filename)
        FXMessageBox.error(self, MBOX_OK, "Error", "#{error}")
      rescue EOFError => error
        File.delete(saveDialog.filename) if File.exist?(saveDialog.filename)
        FXMessageBox.error(self, MBOX_OK, "Error", "#{error}")
      rescue StandardError => error
        FXMessageBox.error(self, MBOX_OK, "Error", "#{error}")
      end
    end
    return 1
  end

  # Action when the button 'Run' is pushed.
  # Let the Parser evaluate the current contents of the editor window and display
  # the corresponding actions of the Turtle.
  #
  def btnRun(sender, sel, event)
    replaceTurtle
    @statusbar.statusLine.text = "Please wait. Parser is busy."
    @lineparser.parse(@editor.text)
    refreshTurtle
  end

  # Action when the button 'Help' is pushed.
  # Open a LogoGUIInfoBox message window containing helpful information.
  #
  def btnHelp(sender, sel, event)
    infobox = LogoGUIInfoBox.new(self)
    infobox.execute(PLACEMENT_SCREEN)
  end

  # Evaluate the contents of the input line
  #
  def lineEval(sender, sel, event)
    @editor.setText(@editor.text.chomp+"\n"+event+"\n")
    @cmdline.setText("")
    @editor.makePositionVisible(@editor.getLength)
    @lineparser.parse(event+"\n")
    refreshTurtle
    @tortoise.clearMessages
  end

  # Enter the given string into the read-only output window.
  #
  def writeLog(messages)
    messages.each do |line|
      @terminal.appendText(line+"\n") if line =~ /ERROR/ # show only error messages
      @terminal.makePositionVisible(@terminal.getLength)
    end
  end

  # Draw the Turtle to the drawing area. The Turtle has the shape of an arrowhead
  # and is drawn in it's current pen color.
  #
  def drawTurtle(drawarea)
    if @tortoise.visible
      drawarea.foreground = @tortoise.color
      # rotate and move turtle to adapt to drawing area coordinate system
      dr = @tortoise.direction + 180
      x0 = @tortoise.posx+2500
      y0 = 2500-@tortoise.posy
      x1 = x0 + 20 * Math.cos((dr-45) * DEG)
      y1 = y0 + 20 * Math.sin((dr-45) * DEG)
      x2 = x0 + 20 * Math.cos((dr+225) * DEG)
      y2 = y0 + 20 * Math.sin((dr+225) * DEG)
      x3 = x0 + 20 * Math.cos((dr+90) * DEG)
      y3 = y0 + 20 * Math.sin((dr+90) * DEG)
      begin
        drawarea.drawLine(x0, y0, x1, y1)
        drawarea.drawLine(x0, y0, x2, y2)
        drawarea.drawLine(x1, y1, x3, y3)
        drawarea.drawLine(x2, y2, x3, y3)
      rescue RangeError
        @tortoise.log "ERROR: Turtle coordinates out of range"
        writeLog @tortoise.messages
      end
    end
  end

  # Replace the Turtle with a new object instance and associate a new
  # Parser object to it.
  #
  def replaceTurtle
    @terminal.setText("")
    @tortoise = LogoTurtle.new(0, 0, 0, "black", "white")
    @lineparser = LogoParser.new(@tortoise)
    @scrollwin.setPosition(-2210, -2210)
  end

  # Update the drawn image, message log and status line information. Check if
  # the color attribute of the Turtle is a compatible color string and replace
  # it if necessary.
  #
  def  refreshTurtle
    # GUI defaults to 'black' for unknown color names. Let's tell our turtle
    @tortoise.color = "black" if fxcolorfromname(@tortoise.color) == 0
    # Default background should be white. Catch invalid names and replace
    @tortoise.background = "white" if @tortoise.background != "black" &&
      fxcolorfromname(@tortoise.background) == 0
    # don't show color if pen is up
    @tortoise.pen ? currentcol = @tortoise.color : currentcol = "NONE"
    @turtleinfo.text = sprintf("Position: %.2f, %.2f  Direction: %.2f  Color: \
#{currentcol}  BG: #{@tortoise.background}",
@tortoise.posx, @tortoise.posy, @tortoise.direction)
    writeLog @tortoise.messages
    @turtlearea.update
  end

  # Returns a PNG image
  #
  def geticon(app, name)
iconimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\
\003\000\000\000\273 H_\000\000\000\360PLTE\000\000\000\377\377\247\377\377\020\
\377\377\357\377\377\356\377\377;\377\377\177\377\377\220\377\377\257\377\377\
\217\377\377\370\377\3774\377\377\304\377\377\021\377\377\336\377\377o\377\377I\
\377\377B\377\377\352\377\377\237\377\377\025\377\377h\377\377\230\377\377\374\
\377\377\344\377\377\367\377\377:\377\377P\377\377\004\377\377A\377\377x\377\
\377X\377\377\363\377\3773\377\377\364\377\377\351\377\377\275\377\377 \377\377\
\256\377\377\331\377\377Y\377\377\337\377\377p\377\377\305\377\377\f\377\377\
\246\377\377\276\377\377\266\377\377\207\377\377\210\377\377\373\377\377\026\
\377\377w\377\377\330\377\377Q\377\377\200\377\377\032\377\377\314\377\377\345\
\377\377\003\377\377\313\377\377J\377\377_\377\377\227\377\377.\377\377g\377\
\377\a\377\377`\377\377'\377\377\e\377\377&\377\377\010\377\377-\377\377\000\
\377\377\240\377\377\322\377\377!\377\377\v\377\377\321\377\377\265\347\223c\
\240\000\000\002\020IDATx^\215\212Ub$I\f\005\243\271\333\314\314\314\36603.\337\
\3776\023)\345\264\313\177\eJ=P\025\177\374O\330\332\332z\363&\236h\016\324\244\
\250\216\360Y~\304H*X\363\022\022\316?\205\253\253\272e\020\233\344'Me\272=\335\
n\267\247%\\\241=\355\213\254I(KKOb\022\023<Yr\275\224\243\236\211o\225\227e\
\225\227q!.M8\376;\370m\307\220\032\305\263/\340\373\327J\rdF\251\347\204\237c>\
\026\001\203\t\242*\025&'\367\367'Ew&\211\340\tM\214i\364d#\337Fo\003z\252\251g\
\322\304^\022\363\363\363\333\256\350\333\214[Fw;\225ws\r`n.\017\231m\261\302\
\346sG\302y.\233n\224r\313\242s\266\273\353sU0%V\333Y|\312\366\245\301\270\274o\
V\233\217\345\345\345O\276b\020\256$\020\347\354\\\004\263\263\263\027\263h\371\
J\215\303}\345\351\316\323\n\354$Y\375\000\252\275(/\244\037C\277\237\021^\364\
\305DZy\334\335=r\004\036\231\254`4X@\317\314`0\030\371F#\334\201\300\310\365R\
\316\2509|\250\200\262\262R\035V4\233\236\260\030\374\271H\350\"\204\215]Kh%p\
\335j]k-\255\254I\352\301\307\311\360D\206h\303\023\264\241\251\000a^\034\375V\
\376\273%\\m\360\360\306\315\221\334\340\036\241F\253F^\26315u95\005\032\212\
\026\242\232 \344\262t\336\026pA\221WnM\257@s\205\277\004b\e\374[\035b\255<\023\
\236\235\202~\352\250e\177G\274S\0223\3473p\0163\006W).\325\300gfm\3555kh\r\274\
\335G\277aga\001Y8t\342\035fr$\034\361\257N\a\350v;e*\343\024\327n\027\350v8\
\000\366\366\016\352{`N6`\217u\230\230\230Xo\274R\327\365\330\034\230\340\361\
\352c\337\352j]\233\250\331\262\227\370\v3\312\325\206\207Ph\357\000\000\000\
\000IEND\256B`\202"

newimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\003\
\000\000\000\273 H_\000\000\000\333PLTE\315\315\315\217\257\273w\234\252\270\
\317\326\231\270\303v\233\251\333\347\353|\240\256\226\265\300s\231\247y\236\
\253\221\261\275\372\373\374{\237\255\230\267\302\241\275\307\335\351\354\364\
\370\371\316\336\343\320\340\345\214\255\272\354\363\365\326\343\350\276\323\
\332\340\353\356\224\263\276\344\355\360\256\307\320\207\251\265\177\242\260\
\273\320\330\211\253\267\330\345\351\251\304\315\213\255\271\361\366\370\351\
\361\363\204\247\264\261\311\322\313\334\342\301\325\334\247\302\313n\224\243\
\274\321\331l\222\241\201\244\261i\217\237\341\353\357\306\331\337f\215\235\266\
\315\325\323\342\346c\213\232b\212\231\263\313\323\367\372\373\311\332\340\347\
\357\362\254\305\317\357\364\366\234\272\304\303\327\335\374\375\376\243\277\
\311\203\246\263p\226\245e\214\234\377\377\377^\206\226q\227\245k\221\240\246\
\301\313\236\273\306\322/\351'\000\000\001\202IDATx^\205\323\347\252\3440\f\200\
\321\330\351\275'\323{\357\345\366~\267\277\377\023\255\271\221 \e\022\355'\201\
\0358\370\307\300H\222\304_\024\003\362\332\"\273m\177\305l\256I\345^\034\347\
\255\266\357-\343\037\3519\323\206\344\221]\226\206\2236\264x\212XV\206\235\372\
f\276\347\347l$a\355&\370h\317\207^w($\302V}G\333\3553e<Ai;Z}\00177\347\233\003\
\337i\010\263\372\264\335\206u#m\264u\v\310\234\270\241U|\215E\241\005/\336?4\
\264\\~\035o\360b\" \231\003\220\205\257d+\204I\230\223m\357-\200SY\226\ab\252\
\213GX\206wb\252\213\267g\200\223\347;\272\024a\332%{\232\001\034\316\306d\353\
\016\a\330y\247;\002\354\375\a\372\001\276\030(d\336\025\340G&\376z\206\230\352\
\342\021\001\234\307\342\203\211\251.\336\036\360\305\345\204n\205\360D\273\341\
\026a\336#\233_\324\002\356\345\017\262\375\002`\177\241\353z_Lu\361\030#\\\213\
\017KLu\361\366\016P\367UU\345b\252\213\207\202\3200\311~\032\000\335\344\223l\
\223\230\005\264\346\277\350z\010u\332\035\366\000\271y&\373f!<\334\220\335\232\
\000\325\333\037t;\200n\020N\323\331\343\261\025\214\2628Z\236^W\371@\276,\272\
\343\365o\377\217\342\265\031\343\0054\\Sm\376\021\305\362\201P\177\001\274J\
\260\322H\277\356\206\000\000\000\000IEND\256B`\202"

loadimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\
\002\000\000\000\003\234/:\000\000\006UIDATx^\255\226\313o\\5\e\306m\2373\231\
\311e\322\334:\241\241 @b\201\220\330 \361\327p)\354XD\005\261a\201\330|\v\340\
\373\204(*\v\020\202\r\v\026\210\025\273R\t\245\271\251m\246\364\006DMU\204\312\
%\275)\311\344>\307\257\375=\357k\3179\tM\346\260\340\211\317k\267\232\361\317\
\317c\373$\252\331l^X\270\372\313\315?o\336\272}\353\326o7\226\226\256]\273\326\
l.@\263\263\263333sss\347\316\235k\376\333J766&\036{\272\257>\252\225JR\243\265\
2\332\367\327R\3244\321\250\344\274\315\262\265Vkgg\a\037\266\226\266\267\267\
\210\310\213\234s\003\003\003\352!%IBD\352p\245x\234\303\f\212\274\313\332N{\
\345\224\332\332uX\207\326\272\277\226$FU{\252\215FC\355\027\211vww\327\327\327\
\261 \f\254\265\250\355v\233\227R&=55\3258\366d}h\334y\020\025V\340\321\241\312\
8\210\234\352\255\232Z\217\251\244z\260/Q\207\v\016\260\216\345\345\345\355\355\
\355\356\2165\342\256\016\214\035\031y\004\251E\252 \245!\3478vh\316o\356\270\
\027\236\201\233r]\271r\245;\330 %\320\264\322\271?8w\334@\215<\e\e\326Y\216\
\364\274wN\225)\025\333\022\254\312\315yf+\306{\347\301\016\315\363\243\272\347\
,\225\263+\005\e\aQ\021&\201\344\024yE\204\201\347A\370\177\362^\233\315\255-\
\334\267\265\2655\265O\021\346PE\350\313\301\002s\221*!\v[\360\2351\017$\t\"{\
\377\336\275\305\305\305\237\256_o\265Z\021),yP\245\tw\364\364\351n`|1\202A\215\
\v\217H\362\202\247\016\2368\034\\P\255\365\312\352\352\352\312*\276/P\210\213\
\313\375\262\343\022v\312\223\271\0302\a\236\357q'|'cB\347\311\221\365I8\017\
\332\263T\270\t>\027\207\020\017W\316~09y\200c<\260\262/d\212\216\271Q\244\266\
\333Y\226YK\024>\3039\311\236r\307\332\267\307\271\343.\276\315\310\350(Sc\260\
\373\220\334\274\267\004\252m3\271M\224\205T\331g\\A\261\2711e\251\n\352\312N[\
\255\365\244\257\356\230\335\tV\305q\210\035F3\301\032\r\177\364\370\027_\206\
\255\255\327\353zh(\221T%\341\242\340\031\271{'\270>,s\3438<\235\237\344\374\
\362\210u\241f\\v\205MD\262\217\261\345\342a\241`\200\271]|\el\234S\2220)+0K1m\
k\t0\274\366C\203o\262Y1{\276\204\203\350J\324\205\235:\000\275aL<\035\361^\
\301\e\250Y\233\177 \240]\242\023O\222\2450\034W\025)9+\364(%\002\222(f[\334Z\
\313\3246\v\206\301\204v\230\356\234-\010\302/\354\252\302w\027l\276\315ig\217\
U\016\206]k-\230b4CAC\344F\231D\005\233\301\361\236Xs\3231|\2475\352aT\001[\313\
`X$\213\034\371\323\240\002\025$\310\266\315X\2116\025\343\227^|)1\306Zj\2147\
\252O<e\311\026\316\003\233\017W\266r\377\356\320\251S\207Q!\323?P\227\027\203\
\023\307>\263\324\241r/'\332Z<\324F\nD\326\271\210)\336\317\334\241\305\a\302\
\264]\274\006\245IZ\341\363L\3410\273\300D\260Y\e@4t@\212im\\\232\310y6L\"\346Q\
\270\307.\304\037\343.\245B\0063:\237\220S\026\324\214\221\314\203(\023\036\267\
LL\223\265\201\346\271)a\a\353\016\225\242\365X\273R\213{\254-Q\006\301\036\253\
\035\233\000\243o\t\337r\324>(8\216i\347\303N\355N\205R\370\360\360mI0\002g\005\
jl@\032\343\373{{RMZ\371\310\212\f\3117/\361\232\353\256T\001cb\245\364\336\275\
\f\251F\2748\255\244f|\244>\334\313\177kj\245C\226(a\t92Gk\260\313\224\246ie\
\307f\230  \303\326\002\211J6fPI\322\355\255\315\343Cu\314\027\366\017l\"\n\
\326\005'\364\374G\253R\231J\245\306\310<\347\"d<\030\205\303\305\027I\034\252\
\260\273hXr\364\314\362\305>+\356\312\035\223\303\224\326xNw_\223\234;\031(\212\
\177\020\306l'&\036m\214\217cq\276xW\027#\324r\260#\026\234\010\017\245\203\244\
\200\227\226\300\245l\255\260{z\252G\e\215\020x\210V\3647dI\334\206<\320 D\036!\
\333\350\230\231\235\253e\231Q\374\315\254\235H\374K\037:\251JR7&)sl\035'\206\"\
\214\310\2149\243eP\265\222\344T<\341\027\200\337\3437\366\342\230\270/?\330\
\251\345\303I$H(\356\253x\rK@\034JU\205\e/\021\034\263\255|[\037\212\032\003\
\255\250$j\304\263\271\321\302,d\345\025\3155\177gq\362^\305\371\3627\006_e_\
\034f\021\367DE3&-\001\037?66V[\177p\347\217\314:\231\235\217\271\245,\254 \304\
\311h)b\323\301p\360\036\320D^`\024\320\230\203\376\311u:2X\177\376\271\372\370\
\330\275\037\027\177_\331T\325j\325\001 N425\tff\274\314*\342\250\343\241\332\
\243\360/\305\330x\032J\300\241;>qtp\240z\373\257\225\231\346\222OkZ+\366oR\r\
\264N\224I\372\006\372\207\207\353\255\365-\273\275\213I-\261|!\201\241\v\a\300\
IW\016\026\r\016\016>;886\\\277x\365\306\317\277.\367\366\327\265\326\230N\002\
\320}\275\265\261\243\243\325Z\r\316{k5k\267\274\323\310\003\260\220D~\235+\225\
Ta\341\245j\036\244o\276\375\356\265\311w_>\371\336\253o\375\357\304\233\037L\
\276\363\361'\237\177\335|H\323\323\323\363\363\347\027\026.\235\277piz\346\342\
\364L\363\207\251\vg\316\316\317\3177/_\276\334\354*\200\017\326\231\357\317\
\376\367\324g'N\376\347\2257\336\177\375\355\017?\372\364\253f\231\260\216\331\
\3319\031\226\353\377\010\310?\2004?A]\000\000\000\000IEND\256B`\202"

saveimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\
\002\000\000\000\003\234/:\000\000\003NIDATx^\305W\315jT1\024>\347\344*\010.d\
\260\257\340\003\264L\005\301\a\230\205\277\213.\n\342\003\371\267\350\v\330\
\2058\364\025\272\020Z\020\334\314J\267Rp!\263\031\367\3429G\2233\271\347\206\
\244\016#\364\372M8\375\222i\362\345\374$a\020\306\305b\2610b\302\037\000\250\
\325\366NNNB\027B\350\002\321\037\323Q\374\004\n\t\224\260\376\203\330\366a6\
\233-\227\027\000\230\333u\323\356 B\000\024Z`\341\203\247\a\320\302\371\331YZH\
\000a\2727mx\346\320\324(\361\237\273\273Q\273\027\246\246\274\260\314\347\363\
\316\034\f\t]\362>t,\002k/q2\231\254V_\322>(\331\035( \000\350\213'm\023\346\
\274\251\000 \266\307,\314\200)\212\210\n\030\371/\216aB\2064\010f|u0[B\335\275\
\f\023V\233\226\254\201\214\263\312\263\303Ch\341\364\364\024\263\210\257\333L\
\231{\345\332\275\307T\356\316C\375\366\3708P\350R\244\251\v\235\321.\2100`\202\
O\224\332\343\345ri^\r,ea\237`;\360|\260\210\305\222-\260\214\fl[M\202\314e\"Mu\
S\250\305\205\263j\000\320a\000D\004\301\026\214J\366Yo\005@;+\000,sLQJ\024\t\
\213,\024\004\310\243\224\254\221\314A\231ED%Zf\341\010\0213\375\210D\267\313\
\271\240 \252:\020\346\201\2046\317\261\021\253\2058\e\a^[\2209Y\203\230\377U\
\250\205\025I\211\240<\256\330\010uy\030\202\255\255\"l^\344\360\202E\e#\214t\
\314\253\325\n\340\016\f *\244*\022\274\252K!\223\231\003\204\304\311[\034\231\
\035\035\035\021a\177/\206@}\201\333m\342\367J\342\353\016\005\nH\321\322\376t\
\037\340\273->\260\023\023~\357\252\276\203\220\354\f.\207\335\275\000\327R\373\
\001m|31\227\210\344\266\t\277\e8\212Y\273\334\201s'U\227\312\346J\325\370\216\
\345\340I\316\001\266|\322\006\367.\376-\036N\260\034\177a\302\217\243\331\274D\
=\350[\331R\376\245\t?\252<s\342_m\r\275|\344\225\t?l\206\272\035\371-\200\225\
\243=ym\302\017\374\237\376M\240\375\024b]\a\231\277\031>\213\264\200\347e-z\
\341\326\325\211-\342\274j\004\021\004_\343\222\271\313IX\372\207M\353+\264l2 \
\340#\006\377\026\252u\372I4\\V\213i%\331\324\\\276\241\352\032\346a\371H\344\
\231{\252p9.\020%?\237\352/\273\027\247\2707\305CME\317\205\251\177q\026\2109s\
\355\034\273X\213\240w\213*U\027\206*\324UFaC\300\r\005iW\303_CMSU\270j\340g\
\213\002\271p\264\#@\335cO\312\010H\252\275\260\027\346'D\254\253\251~\330\332\
\304ymo\254\363\310.<\274\f\356\252\216\354\261\216\032\352\362\254\353X\302Z\
\377\222\200\2179\307U\246\333Y\f\315|\267f\335\324\336=.s\234p\357\312s\314\
\000\241\024\036\t\322\374\231:\276p\306\371\340\034GR\245\n2\241-\217\370-\325\
\366\353d\362\367G8\307\377\027\277\001\333D\273\323I\345\226\326\000\000\000\
\000IEND\256B`\202"

runimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\003\
\000\000\000\273 H_\000\000\000uPLTE\315\315\315\000\000\000\000\252\000\000F\
\000\000g\000\000H\000\000_\000\000L\000\000c\000\000P\000\000U\000\000\324\000\
\000\317\000\000\311\000\000\303\000\000\275\000\000\270\000\000\263\000\000\
\256\000\000\374\000\000W\000\000\371\000\000Z\000\000\\\000\000\360\000\000\
\353\000\000J\000\000a\000\000\346\000\000\340\000\000N\000\000e\000\000\332\
\000\000h\000\000S\000\000\377\000\000E\000\000\365\000\000i\000\235\0161\320\
\000\000\000\266IDATx^\255\324\327\256\2030\020\006\341\f\244\367\336{r\316\373\
?b\020A+\212\355\337\027\371\256G\010\360z[\361 6|G\246$I\032\225\262\314t\320)\
\213\334Q\246\314\v=\22123\327`\312\264d\020H\231T<\275)\343\232\241'e\324\260v\
\246\334\035\266\216\224\213\323\256\221r\366\350\326R\016^\247J\312>\240_J\371\
\v\272Y\312Fh\027)+\351\221\247\274\244\377o\030\365\274\f\372\r\277\220\337\\@\
\374E\2038\027\2038i\203\230\035\203\230F\203\230o\203\2701\006q\a\r\342V\e\304\
\2360\210\315c\260]&`\333Q \205\330e\377S\037\034\r:\373~!\263k\000\000\000\000\
IEND\256B`\202"

helpimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\
\000\000\000\000\251\225\347\261\000\000\003\032IDATx^}\324]lTU\030\206\321\247\
\273=\235\302\351\217S\nq\206\210vj bD\021!\332fL\323\"\020:\320fb\r\021\n\261\
\021\230\030IM \321\246!`lT.$*\332\230\210RM\224\004h\025\323i\255N\031#A0\344\
\213)&\030I\346\302\e\315@5\024\265t\nS\305\2753sv&\304u\363&_\336\354\213\223\
\234\027\261\215\036\332\331\034\252v\335\352PS\354\355Q\261\025\ty\227FN\3772K\
\216\272\273a\335}x$\347\263\315~\000\345\006\227,\t\272\n\240\374\251\343\222\
\223/\356\r\000\316\362X\337\300\350\330\330\350@_l\205\003\314\177\251\260\270\
\t\250h?\374\215x\016\267\225\003\321\204]L\264\000\215\375R\340H#\360\304\327^\
1\321\f\3051\271\215]\0164&\362\305\215\340\356\023m\354\235\035\221\306pK\347\
\253q\321z+\2415W\354\001\247W\264\327\227*\214`\247\371\216o\270\320m\212\0035\
\320\245\217\311m\n\317\303\203\372\370\"T\017\352b;4\213\026\303\306\n\363f\
\023DE\024\027\207\250\332\001\300\317\375\330\220/ul\257$>\016\362$l\021\355Y\
\264\332\350j\amiR\3377CTT\3724e-\000\\;\247\343\336\267z\016<\203\226\232\320\
\021)\343LZ\375\224\346\376\305\000L^\322\261z!\204\315\2233\277\352X\274\234\
\364Eu\001V\242]\017\314Q@\020\230\314\242\315\242=\n\027T\212\022S$\360\376\
\307\207^~nS-p\3264\234\273\320\036rHQKU\\\n\034\234\213\026J\210\026\257\342\
\036u\225\371>lp\264g\nm\203\037\315W\315dI\206\362\022l|r\020\243~=Fq\005\031\
\005\331\177\260\375\330\207\261\252\273\006\243H\201\3621\225\3056<\215\326\
\326\e\304 \373\027>U\305\3573X\256\216\353(\335\275w\0369\231+T\252\205\374}\
\031K\3467\035\035O\343\231\230\"\250B\3348\217%;\r\340\254\3032~\203\272\222ep\
\276\023O\331\276\233@i\r\226\357\341\001\206\0270\367\230\374\257cs\250\031B\
\332`\253xN\275\327w\313\aI\361l\203VQl(\345d\212\274?\367<\177\313\376\031\362\
R')\215\200\376\a\327H\336\347e\000\001k\241\232 \"\242\240c\036_\235 gV\001\
\024\221w\342\024\376\255\240 \024\2037\277\305@a\203\263\357\302\316:s\217F\
\270\376\312w\030\263\323\000\031\f\316\355\277\306\372vL\221\2560\177t\017\242\
9\r\253\376S\214\366E\367\004\217uy\3738R\017\264\016H\201\343\255\300#C\366\
\354%6\002\201\355G\305\362i\347\002`\355H\301\220\276P\t\270\341=\375\361D2\
\231\210\037\331\375\270\v\224\357\262\207\324\370h\255\017\300\361\327=\270,t\
\207\003\3404\177x\333\261\377a\370L\032\317\235\365kV\342\021[\374\300\226\206\
E\376\n\327\277(\334\361Z\\l\377\002\226\240&U\344=:h\000\000\000\000IEND\256B`\
\202"

exitimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\
\003\000\000\000\273 H_\000\000\000\253PLTE\315\315\315\372\000\000\356\000\000\
\367\000\000\234\000\000\246\000\000\222\000\000\217\000\000\362\000\000\351\
\000\000\345\000\000\261\000\000\255\000\000\230\000\000\337\000\000\311\000\
\000\304\000\000\212\000\000\225\000\000\242\000\000\270\000\000\275\000\000\
\264\000\000\277\000\000\340\000\000\236\000\000\251\000\000\312\000\000\326\
\000\000\377\000\000\332\000\000\352\000\000\375\000\000\317\000\000\363\000\
\000\324\000\000\240\000\000\215\000\000\303\000\000\253\000\000\206\000\000\
\216\000\000\226\000\000\204\000\000\211\000\000\344\000\000\376\000\000\273\
\000\000\334\000\000\321\000\000\306\000\000\346\000\000\260\000\000\366\000\
\000\267\000\000\360\000\000\200\000\000\030h\216o\000\000\001mIDATx^\225\224\
\325\216\3040\fE]ff\034f\306\335\377\377\262\235\251U\225a\357K\022\373(jc_\303\
\177\224$\303y\\\373H\314\343\332K\026\371\244\217,\322C\344\271\224DrJ\264q\
\0241-_\222q\324\274\311\375\3143\262\010|\270\027ON\352\334cB\362/bZ\016\021\
\024O\n\242_\345\256\276(\220<U\r\316I\341\244\246q9t\213S\365$\220P\323DT\323\
\305,*\002\367h\266HU\021\032\362?\234\246{\371\361\342\351\332\207\204\026\305\
3m\355\330.\036\236\256\355\254\265\031\264*\322\235\267$\207\337-\023\312\322\
\333\321\241C\236-m\030+\000\240\003\213\331H6t\312\225\231%\255\230\246\251\
\320KF\206\036\205\026\275g\271\035\307\356i\vz\025(\354\3128\032+V\201\001\231\
\234\261=l\r\016\006\301\335\361\360{8\356\2068:\277\321\354\347\230\342\e\203>\
\356Y\376\353\260\233\273\340;\002\340;\272]\334\035+\363\335be\274v\356\206\
\265\306\003\326:j\343\256\330=\371\021\273'nr\017\354\307\"\200\375\3507\374\
\206\035^\016a\207\327\034G\241g\252A\364L\305\305gt!\324\204.$J\266F_CC\350\
\353bP\340\244\200\026\341\244Hr.#\241U\031\207\344\270\2617z\220\216\035\315\
\343\207\375x\375\001\362T,\010[5Q\017\000\000\000\000IEND\256B`\202"

    case name
    when "icon"
      FXPNGIcon.new(app, iconimg, 0, 0, 40, 40)
    when "new"
      FXPNGIcon.new(app, newimg, 0, 0, 40, 40)
    when "load"
      FXPNGIcon.new(app, loadimg, 0, 0, 40, 40)
    when "save"
      FXPNGIcon.new(app, saveimg, 0, 0, 40, 40)
    when "run"
      FXPNGIcon.new(app, runimg, 0, 0, 40, 40)
    when "help"
      FXPNGIcon.new(app, helpimg, 0, 0, 40, 40)
    when "exit"
      FXPNGIcon.new(app, exitimg, 0, 0, 40, 40)
    else
      nil
    end
  end
end


